#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Interface;
use strict;
use warnings;
use NEXT;
use base qw(Opsview::Web::ControllerBase::DrawRRD);

__PACKAGE__->mk_accessors(qw/model_name/);

__PACKAGE__->config( model_name => "RuntimeDB::Host", );

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->forward("/authenticate");
}

sub object : PathPart('interface') Chained('/') CaptureArgs(2) {
    my ( $self, $c, $name, $interfaceid ) = @_;
    my $it = $self->model_name->search( { name => $name } );
    unless ( $it->count ) {
        $c->stash( error => "Name $name not found" );
        $c->detach("/soft_error");
    }
    $c->stash( object      => $it->first );
    $c->stash( interfaceid => $interfaceid );
}

sub end : Private {
    my ( $self, $c ) = @_;
    if ( $c->stash->{rrderror} ) {
        $c->stash( error => $c->stash->{rrderror} );
        $c->detach("/soft_error");
    }
    $c->forward("/end");
}

# Create the general rrd options here
sub rrd : Chained('object') CaptureArgs(0) {
    my ( $self, $c ) = @_;
    my @action = split( "/", $c->req->action );
    $c->stash( type => $action[-1] );    # bits, autil, etc

    my $host        = $c->stash->{object};
    my $interfaceid = $c->stash->{interfaceid};
    $c->stash( node => $host->ip );
    $c->stash( ifid => $interfaceid );
    $self->get_nmis_info($c);
    my $database = $c->stash->{rrddatabase};
    if ( !-e $database ) {
        $c->stash( error => $c->loc( "No rrd database for [_1], interface id [_2]", $host->name, $interfaceid ) );
        $c->detach("/soft_error");
    }
    my $speed    = $c->stash->{speed};
    my $rawspeed = $c->stash->{rawspeed};
    my $ifdescr  = $c->stash->{ifDescr};

    my $end = time;
    $c->stash( end => $end );
    my $start = $end - ( 48 * 60 * 60 );
    $c->stash( start  => $start );
    $c->stash( width  => $c->req->param("width") || 400 );
    $c->stash( height => $c->req->param("height") || 80 );
    $c->stash( title  => $host->name . ": $ifdescr - from $start to $end" );
    my $rrdoptions = [ "--title", $c->stash->{title}, "--start", $c->stash->{start}, "--end", $c->stash->{end}, "--width", $c->stash->{width}, "--height", $c->stash->{height}, "--interlace", ];
    $c->stash( rrdoptions => $rrdoptions );
}

sub bits : Chained('rrd') Args(0) {
    my ( $self, $c ) = @_;
    my $database = $c->stash->{rrddatabase};
    my $vlabel = $NMIS::config{graph_split} == 1 ? "% Avg bps" : "In(-) Out(+) % Avg bps";

    my $weight  = 0.983;
    my $options = $c->stash->{rrdoptions};
    push @$options, "--vertical-label", $vlabel, "DEF:input=$database:ifInOctets:AVERAGE", "DEF:output=$database:ifOutOctets:AVERAGE", "DEF:maxinput=$database:ifInOctets:MAX", "DEF:maxoutput=$database:ifOutOctets:MAX", "DEF:status=$database:ifOperStatus:AVERAGE", "CDEF:inputBits=input,8,*,$NMIS::config{graph_split},*", "CDEF:outputBits=output,8,*", "CDEF:maxin=maxinput,8,*,$NMIS::config{graph_split},*", "CDEF:maxout=maxoutput,8,*",

        # moving avg calcs should be on average in/out, not max values,as granularity is lost for bigger slices.
        #"CDEF:inp2in=maxin,UN,1,maxin,IF",
        #"CDEF:inp2out=maxout,UN,1,maxout,IF",
        "CDEF:inp2in=inputBits,UN,1,inputBits,IF", "CDEF:inp2out=outputBits,UN,1,outputBits,IF", "CDEF:mavgin=PREV,UN,inp2in,PREV,IF,inp2in,-,$weight,*,inp2in,+", "CDEF:mavgout=PREV,UN,inp2out,PREV,IF,inp2out,-,$weight,*,inp2out,+", "LINE2:inputBits#0000ff:In bits/sec", "LINE2:outputBits#00ff00:Out bits/sec",

        #"STACK:inputBits#aaaaaa:Agg In and Out bits/sec\\l",
        #"LINE1:inp2#FF0000:Max values\n",
        "LINE3:mavgin#0000AA:Input Moving average, weight $weight", "LINE2:mavgout#00AA00:Output Moving average, weight $weight\\l", "GPRINT:inputBits:AVERAGE:Avg In bits/sec %1.2lf", "GPRINT:inputBits:MAX:Max In bits/sec %1.2lf\\l", "GPRINT:outputBits:AVERAGE:Avg Out bits/sec %1.2lf", "GPRINT:outputBits:MAX:Max Out bits/sec %1.2lf\\l", "HRULE:" . $c->stash->{rawspeed} . "#ff0000", "GPRINT:status:AVERAGE:Avg Availability %1.2lf", "COMMENT:Interface Speed " . $c->stash->{speed},;
    $self->rrdgraph($c);
}

sub autil : Chained('rrd') Args(0) {
    my ( $self, $c ) = @_;
    my $rawspeed = $c->stash->{rawspeed};
    my $speed    = $c->stash->{speed};
    my $database = $c->stash->{rrddatabase};

    my $vlabel = $NMIS::config{graph_split} == 1 ? "% Avg Util" : "In(-) Out(+) % Avg Util";
    my $options = $c->stash->{rrdoptions};
    push @$options, "--vertical-label", $vlabel,

        # Might be a good option on an active interface.
        #"--logarithmic",
        "DEF:input=$database:ifInOctets:AVERAGE", "DEF:output=$database:ifOutOctets:AVERAGE", "CDEF:inputUtil=input,8,*,$rawspeed,/,100,*,$NMIS::config{graph_split},*", "CDEF:outputUtil=output,8,*,$rawspeed,/,100,*",

        #"CDEF:totalUtil=outputUtil,inputUtil,+,2,/",
        "LINE1:inputUtil#0000ff:In % Util", "LINE1:outputUtil#00ff00:Out % Util",

        #"LINE2:totalUtil#000000:Total % Util\\n",
        "GPRINT:inputUtil:AVERAGE:Avg In %1.2lf", "GPRINT:outputUtil:AVERAGE:Avg Out %1.2lf",

        #"GPRINT:totalUtil:AVERAGE:Avg Total %1.2lf",
        "COMMENT:Interface Speed $speed",;
    $self->rrdgraph($c);
}

sub pkts : Chained('rrd') Args(0) {
    my ( $self, $c ) = @_;
    my $rawspeed = $c->stash->{rawspeed};
    my $speed    = $c->stash->{speed};
    my $database = $c->stash->{rrddatabase};

    my $vlabel = $NMIS::config{graph_split} == 1 ? "% Avg Util" : "In(-) Out(+) % Avg Util";
    my $options = $c->stash->{rrdoptions};
    push @$options, "--vertical-label", 'Packets/Second', "DEF:ifInOctets=$database:ifInOctets:AVERAGE", "DEF:ifInUcastPkts=$database:ifInUcastPkts:AVERAGE", "DEF:ifInNUcastPkts=$database:ifInNUcastPkts:AVERAGE", "DEF:ifInDiscards=$database:ifInDiscards:AVERAGE", "DEF:ifInErrors=$database:ifInErrors:AVERAGE", "DEF:ifOutOctets=$database:ifOutOctets:AVERAGE", "DEF:ifOutUcastPkts=$database:ifOutUcastPkts:AVERAGE", "DEF:ifOutNUcastPkts=$database:ifOutNUcastPkts:AVERAGE", "DEF:ifOutDiscards=$database:ifOutDiscards:AVERAGE", "DEF:ifOutErrors=$database:ifOutErrors:AVERAGE", "DEF:MifInUcastPkts=$database:ifInUcastPkts:MAX", "DEF:MifInNUcastPkts=$database:ifInNUcastPkts:MAX", "DEF:MifOutUcastPkts=$database:ifOutUcastPkts:MAX", "DEF:MifOutNUcastPkts=$database:ifOutNUcastPkts:MAX",

        #Turn the ifxxOctets into Bits per second
        "CDEF:inputBits=ifInOctets,8,*", "CDEF:outputBits=ifOutOctets,8,*",

        #In packets should always include Discards and Errors as use bandwidth
        "CDEF:ifInPkts=ifInUcastPkts,ifInNUcastPkts,ifInDiscards,ifInErrors,+,+,+",

        #Out packets shouldn't as they don't leave the device
        "CDEF:ifOutPkts=ifOutUcastPkts,ifOutNUcastPkts,+",

        #Total Packets in and out.
        "CDEF:ifPkts=ifInPkts,ifOutPkts,+",

        #Max Packets
        "CDEF:MifInPkts=MifInUcastPkts,MifInNUcastPkts,+", "CDEF:MifOutPkts=MifOutUcastPkts,MifOutNUcastPkts,+",

        #Get average packet size stats
        "CDEF:avgInPkt=ifInOctets,ifInPkts,/", "CDEF:avgOutPkt=ifOutOctets,ifOutPkts,/", "CDEF:avgPkt=ifOutOctets,ifInOctets,+,ifPkts,/",

        #Draw some lines and stuff
        "AREA:ifInUcastPkts#0000aa:ifInUcastPkts/sec", "STACK:ifInNUcastPkts#0000ff:ifInNUcastPkts/sec\\n", "STACK:ifOutUcastPkts#00aa00:ifOutUcastPkts/sec", "STACK:ifOutNUcastPkts#00ff00:ifOutNUcastPkts/sec\\n", "LINE2:ifInPkts#0000ff:ifInPkts/sec", "LINE2:ifOutPkts#00ff00:ifOutPkts/sec", "LINE1:ifPkts#ff0000:ifPkts/sec\\n", "GPRINT:ifInUcastPkts:AVERAGE:Avg ifInUcastPkts %1.2lf", "GPRINT:ifInNUcastPkts:AVERAGE:Avg ifInNUcastPkts %1.2lf\\n", "GPRINT:ifOutUcastPkts:AVERAGE:Avg ifOutUcastPkts %1.2lf", "GPRINT:ifOutNUcastPkts:AVERAGE:Avg ifOutNUcastPkts %1.2lf\\n",

        #"COMMENT:\\\\l",
        "GPRINT:ifInPkts:AVERAGE:Avg ifInPkts %1.2lf", "GPRINT:ifOutPkts:AVERAGE:Avg ifOutPkts %1.2lf", "GPRINT:ifPkts:AVERAGE:Avg ifPkts %1.2lf", "GPRINT:MifInPkts:MAX:Max ifInPkts %1.2lf", "GPRINT:MifOutPkts:MAX:Max ifOutPkts %1.2lf\\n", "GPRINT:avgInPkt:AVERAGE:Avg In Packet Size %1.2lf", "GPRINT:avgOutPkt:AVERAGE:Avg Out Packet Size %1.2lf", "GPRINT:avgPkt:AVERAGE:Avg Packet Size %1.2lf\\n", "GPRINT:inputBits:AVERAGE:Avg In bits/sec %1.2lf", "GPRINT:outputBits:AVERAGE:Avg Out bits/sec %1.2lf", "GPRINT:ifInOctets:AVERAGE:Avg In bytes/sec %1.2lf", "GPRINT:ifOutOctets:AVERAGE:Avg Out bytes/sec %1.2lf",;
    $self->rrdgraph($c);
}

sub epkts : Chained('rrd') Args(0) {
    my ( $self, $c ) = @_;
    my $rawspeed = $c->stash->{rawspeed};
    my $speed    = $c->stash->{speed};
    my $database = $c->stash->{rrddatabase};

    my $vlabel = $NMIS::config{graph_split} == 1 ? "% Avg Util" : "In(-) Out(+) % Avg Util";
    my $options = $c->stash->{rrdoptions};
    push @$options, "--vertical-label", 'Percentage', "DEF:ifInUcastPkts=$database:ifInUcastPkts:AVERAGE", "DEF:ifInNUcastPkts=$database:ifInNUcastPkts:AVERAGE", "DEF:ifInDiscards=$database:ifInDiscards:AVERAGE", "DEF:ifOutUcastPkts=$database:ifOutUcastPkts:AVERAGE", "DEF:ifOutNUcastPkts=$database:ifOutNUcastPkts:AVERAGE", "DEF:ifInErrors=$database:ifInErrors:AVERAGE", "DEF:ifOutDiscards=$database:ifOutDiscards:AVERAGE", "DEF:ifOutErrors=$database:ifOutErrors:AVERAGE",

        #In packets should always include Discards and Errors as use bandwidth
        "CDEF:ifInPkts=ifInUcastPkts,ifInNUcastPkts,ifInDiscards,ifInErrors,+,+,+",

        #Out packets shouldn't as they don't leave the device
        "CDEF:ifOutPkts=ifOutUcastPkts,ifOutNUcastPkts,ifOutDiscards,ifOutErrors,+,+,+",

        # Percentage of In Error
        "CDEF:PInDiscards=ifInDiscards,ifInPkts,/,100,*", "CDEF:POutDiscards=ifOutDiscards,ifOutPkts,/,100,*", "CDEF:PInErrors=ifInErrors,ifInPkts,/,100,*", "CDEF:POutErrors=ifOutErrors,ifOutPkts,/,100,*",

        #Draw some lines and stuff
        "LINE2:PInDiscards#00cc00:ifInDiscards", "LINE2:POutDiscards#ffbb00:ifOutDiscards", "LINE2:PInErrors#aa00cc:ifInErrors", "LINE2:POutErrors#ff0000:ifOutErrors\\n",

        #"STACK:inputBits#aaaaaa:Agg In and Out bits/sec",
        "GPRINT:PInDiscards:AVERAGE:Percentage InDiscards %1.2lf %%", "GPRINT:PInErrors:AVERAGE:Percentage InErrors %1.2lf %%\\n", "GPRINT:POutDiscards:AVERAGE:Percentage OutDiscards %1.2lf %%", "GPRINT:POutErrors:AVERAGE:Percentage OutErrors %1.2lf %%",;
    $self->rrdgraph($c);
}

use NMIS;
use func;
use csv;

loadConfiguration("/usr/local/nagios/nmis/conf/nmis.conf");

# This is effectively copied from nmiscgi.pl
sub get_nmis_info {
    my ( $self, $c ) = @_;
    my %args = @_;
    my $node = $c->stash->{node};
    my $type = $c->stash->{type};
    my $intf = $c->stash->{ifid};

    my $speed;

    loadSystemFile($node);
    loadNodeDetails;
    my %interfaceTable;
    if ( getGraphType($type) =~ /interface|pkts|cbqos|calls/ ) {
        %interfaceTable = &loadCSV( "$NMIS::config{'<nmis_var>'}/$node-interface.dat", "ifIndex", "\t" );
        $speed = &convertIfSpeed( $interfaceTable{$intf}{ifSpeed} );
        $c->stash( speed    => $speed );
        $c->stash( rawspeed => $interfaceTable{$intf}{ifSpeed} );
    }

    my $extName = $interfaceTable{$intf}{ifDescr};

    my $database = getRRDFileName(
        type => $type,
        node => $node,

        #group => $group,
        nodeType => $NMIS::systemTable{nodeType}, extName => $extName,

        #item => $item
    );

    $c->stash( rrddatabase => $database );
    $c->stash( ifDescr     => $interfaceTable{$intf}{ifDescr} );
}

1;
