#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::NMIS;

use strict;
use warnings;
use lib "/usr/local/nagios/nmis/lib";
use NMIS;
use func;

use base qw/Catalyst::Controller/;

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    $c->stash( additional_template_paths => [ $c->path_to( 'root', 'nmis' )->stringify ] );
    return $c->forward('/authenticate');
}

sub catch_all : Path {
    my ( $self, $c ) = @_;
    $c->res->redirect( $c->uri_for("/nmis/status") );
}

sub status : Local {
    my ( $self, $c ) = @_;
    loadConfiguration("/usr/local/nagios/nmis/conf/nmis.conf");
    loadNodeDetails;    # TODO: This could be a problem in FastCGI mode
    my %summary = getGroupSummary();
    $c->stash( nmis => \%summary );

    my @group_data;
    foreach my $group ( sort keys %NMIS::groupTable ) {
        my %group_summary = getGroupSummary($group);
        my %old_group_summary = getGroupSummary( $group, "-16 hours", "-8 hours" );

        # Calculate trend
        foreach my $thing (qw(metric reachable available health)) {
            my $iconname = "${thing}_icon";
            if ( $group_summary{average}{$thing} eq "N/A" ) {
                $group_summary{average}{$iconname} = "";
            }
            elsif ( $old_group_summary{average}{$thing} <= $group_summary{average}{$thing} ) {
                $group_summary{average}{$iconname} = "up";
            }
            else {
                $group_summary{average}{$iconname} = "down";
            }
        }
        foreach my $thing (qw(response)) {
            my $iconname = "${thing}_icon";
            if ( $group_summary{average}{$thing} eq "N/A" ) {
                $group_summary{average}{$iconname} = "";
            }
            elsif ( $old_group_summary{average}{$thing} < $group_summary{average}{$thing} ) {
                $group_summary{average}{$iconname} = "up";
            }
            else {
                $group_summary{average}{$iconname} = "down";
            }
        }
        $group_summary{name}          = $group;
        $group_summary{overallStatus} = overallNodeStatus($group);
        $group_summary{overallColor}  = eventColor( $group_summary{overallStatus} );
        push @group_data, \%group_summary;
    }
    $c->stash( group_data => \@group_data );
}

sub graph : Local {
    my ( $self, $c ) = @_;
    my $database = "/usr/local/nagios/nmis/database/metrics/network.rrd";
    my $end      = time;
    my $start    = $end - ( 48 * 60 * 60 );
    my $width    = $c->req->param("width") || 750;
    my $height   = $c->req->param("height") || 120;
    my $title    = "Network stats - 48 hours from " . ( scalar localtime $start ) . " to " . ( scalar localtime $end );
    my @options  = ( "--title", "$title", "--vertical-label", 'Network Metrics', "--start", "$start", "--end", "$end", "--width", "$width", "--height", "$height", "--imgformat", "PNG", "--interlace", "DEF:avail=$database:availability:AVERAGE", "DEF:health=$database:health:AVERAGE", "DEF:reach=$database:reachability:AVERAGE", "DEF:status=$database:status:AVERAGE", "LINE3:avail#00ff00:Availability", "LINE3:health#ff9900:Health", "LINE2:reach#0000ff:Reachability", "LINE2:status#ff0000:Status", "GPRINT:avail:AVERAGE:Avg Available %1.2lf", "GPRINT:health:AVERAGE:Avg Health %1.2lf", "GPRINT:reach:AVERAGE:Avg Reachable %1.2lf", "GPRINT:status:AVERAGE:Avg Status %1.2lf" );
    use File::Temp;
    my $tmp = new File::Temp();
    my ( $averages, $x, $y ) = RRDs::graph( $tmp->filename, @options );
    seek( $tmp, 0, 0 ) or croak("Cannot seek temp file: $!");
    my $data;
    my $length = 0;

    while ( $tmp->read( my $buffer, 4096 ) ) {
        $length += length($buffer);
        $data .= $buffer;
    }
    $c->res->body($data);
    $c->res->content_length($length);
    $c->res->content_type("image/png");
}

1;
