#!/usr/bin/perl

use Test::More tests => 31;

use FindBin qw($Bin);
use lib "$Bin/lib",              "$Bin/../lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";
use strict;
use Opsview::Web;

use Opsview::Config;
use Test::MockObject;

use Catalyst::Request;
use Test::MockObject::Extends;

my $req = Catalyst::Request->new();
$req = Test::MockObject::Extends->new($req);
my $cookies = {};
$req->mock( "cookies", sub {$cookies} );
my $log = Test::MockObject->new();
my $log_debug;
my $log_info;
my $log_error;
$log->mock( "debug", sub { shift; $log_debug = shift } );
$log->mock( "info",  sub { shift; $log_info  = shift } );
$log->mock( "error", sub { shift; $log_error = shift } );

package Opsview::Auditlog;

my $system_message;
sub system { shift; $system_message = shift }
sub system_message {$system_message}

package main;

my $c = Opsview::Web->new;
$c = Test::MockObject::Extends->new($c);
my $stash = {};
my $served_filename;

#my $log_error;
my $loc_message;
my $forward;

#my $config = {};
my $user     = Test::MockObject->new();
my $username = "robin";
$user->mock( "username", sub {$username} );
$c->config->{session}->{expires} = 10;
$c->mock(
    "stash",
    sub {
        shift;
        if (@_) { $stash->{ $_[0] } = $_[1] }
        else    {$stash}
    }
);
$c->mock( "serve_static_file", sub { shift; $served_filename = shift } );
$c->mock( "log",               sub {$log} );
$c->mock( "loc",               sub { shift; $loc_message     = shift } );
$c->mock( "request",           sub {$req} );
$c->mock( "req",               sub {$req} );
$c->mock( "debug",             sub {0} );

#$c->mock( "error", sub { shift; $log_error = shift } );
$c->mock( "forward", sub { shift; $forward = shift } );

#$c->mock( "config", sub { $config } );
$c->mock( "user", sub {$user} );

use_ok("Opsview::Web::Controller::Root");

my $root = Opsview::Web::Controller::Root->new;
$root = Test::MockObject::Extends->new($root);
$root->mock( "authtkt_ignoreip", sub {0} );

my $rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of no auth ticket" );
is( $log_debug, "No auth_tkt found" );

use Apache::AuthTkt;
use CGI::Cookie;
my $tkt = Apache::AuthTkt->new( secret => Opsview::Config->authtkt_shared_secret() );
my $auth_cookie = $tkt->cookie( uid => "admin", ip_addr => "127.0.0.1", ts => time() - 3800 );
$cookies = CGI::Cookie->parse($auth_cookie);
$c->request->address("127.0.0.2");
$rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of invalid address in ticket" );
is( $log_debug,                        "auth_tkt ticket found" );
is( $log_info,                         "Invalid authentication ticket found from 127.0.0.2" );
is( Opsview::Auditlog->system_message, "Invalid authentication ticket found from 127.0.0.2" );
is( $forward,                          "/expire_ticket" );
is( $stash->{login_error},             "messages.authentication.invalidTicket" );

$forward = "";
$c->request->address("127.0.0.1");
$rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of expired ticket time" );
is( $log_info,             "auth_tkt ticket expired" );
is( $forward,              "/expire_ticket" );
is( $stash->{login_error}, "messages.authentication.expiredTicket" );

$c->config->{session}->{expires} = 4000;

$forward = "";
$c->set_false("user");
$c->mock( "find_user", sub {undef} );
$rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of not finding user in DB" );
is( $log_info,             "Username 'admin' via auth_tkt not found in credential DB" );
is( $forward,              "/expire_ticket" );
is( $stash->{login_error}, "messages.authentication.userDoesNotExist" );

$c->config->{authtkt_default_username} = "guest";
$forward = "";
$c->set_false("user");
$c->mock( "find_user", sub {undef} );
$rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of not finding authtkt_default_username in DB" );
is( $log_error,            "Username 'admin' via auth_tkt is valid, but user not found and authtkt_default_username 'guest' not found" );
is( $forward,              "/expire_ticket" );
is( $stash->{login_error}, "messages.authentication.userDoesNotExist" );

$forward = "";
$c->set_false("user");
my $user_object = {};
$c->mock(
    "find_user",
    sub {
        if   ( $_[1]->{name} eq "guest" ) {$user_object}
        else                                  {undef}
    }
);
$c->set_true("set_authenticated");
$rc = $root->validate_ticket($c);
is( $rc, 1, "Returned 1 because using authtkt_default_username" );
is( $log_info,  "Username 'admin' logged in via auth_tkt as 'guest'" );
is( $forward,   "/extendticket" );
is( $log_debug, "auth_tkt ticket renewed" );

delete $c->config->{authtkt_default_username};
$forward = "";
$c->set_false("user");
my $user_object = {};
$c->mock( "find_user", sub {$user_object} );
$c->set_true("set_authenticated");
$rc = $root->validate_ticket($c);
is( $rc, 1, "Returned 1 because user ok (without using authtkt_default_username)" );
is( $log_info,  "Username 'admin' logged in via auth_tkt" );
is( $forward,   "/extendticket" );
is( $log_debug, "auth_tkt ticket renewed" );

# Test for ignored ips
$auth_cookie = $tkt->cookie( uid => "admin", ip_addr => "0.0.0.0", ts => time() - 3800 );
$cookies = CGI::Cookie->parse($auth_cookie);
$c->request->address("127.0.0.2");
$rc = $root->validate_ticket($c);
is( $rc, 0, "Returned 0 because of invalid address in ticket" );
$root->mock( "authtkt_ignoreip", sub {1} );
$rc = $root->validate_ticket($c);
is( $rc, 1, "Return 1 because we're ignoring the request address now" );
