#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use lib "$Bin/lib";
use Opsview::TestUtils;

use_ok("Opsview");

my $mech = Test::WWW::Mechanize::Catalyst->new;
log_in( $mech, 'admin', 'initial' );
submit_passive(193);
$mech->content_like(qr/Change of state for.*submitted/);

# Test that <>; are stripped
$mech->get_ok('/state/service/193/change');
$mech->form_name('main_form');
$mech->field( 'state'   => '1' );
$mech->field( 'comment' => 'warn <script>alert("hello");</script>' );
$mech->click('submit');
$mech->content_like(qr/Change of state for.*submitted/);
$mech->get_ok("/admin/auditlog");
$mech->content_contains( q{/ on monitored_by_slave as WARNING: warn scriptalert(&quot;hello&quot;)/script}, "Stripped data when submitting results" );

log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );
submit_passive(193);
$mech->content_like(qr/Change of state for.*submitted/);

log_out($mech);

log_in( $mech, 'somehosts', 'somehosts' );
submit_passive(193);
$mech->content_like(qr/Change of state for.*submitted/);
submit_passive(216);
$mech->content_like(qr/You are not authorised to change the state of this service/);

log_out($mech);

log_in( $mech, 'readonly', 'readonly' );
submit_passive(193);
$mech->content_like(qr/You are not authorised to change the state of this service/);
log_out($mech);

sub submit_passive {
    my $id = shift;
    $mech->get_ok( 'http://localhost/state/service/' . $id . '/change' );
    $mech->form_name('main_form');
    $mech->field( 'state'   => '0' );
    $mech->field( 'comment' => 'forced for testing purposes' );
    $mech->click('submit');
}
