#!perl
use strict;
use warnings;
use Test::More;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/lib";

plan 'no_plan';

use_ok("Opsview::Web::Utils");

my $params = {
    "managementurl[name]" => [ "SSH",                 "Telnet",                 "other22",   "other33" ],
    "managementurl[url]"  => [ 'ssh://$HOSTADDRESS$', 'telnet://$HOSTADDRESS$', 'http://22', 'http://$HOSTADDRESS$:8081' ],
    rubbish               => "ignored",
};

my $expected = {
    managementurl => [
        {   name => "SSH",
            url  => 'ssh://$HOSTADDRESS$',
        },
        {   name => "Telnet",
            url  => 'telnet://$HOSTADDRESS$',
        },
        {   name => "other22",
            url  => 'http://22',
        },
        {   name => "other33",
            url  => 'http://$HOSTADDRESS$:8081',
        }
    ]
};

is_deeply( Opsview::Web::Utils->convert_multilist_params_to_hash($params), $expected, "Got expected hash with multi values" );

# Test single valued parameters (Catalyst may return in this way)
$params = {
    "managementurl[name]" => "SSH",
    "managementurl[url]"  => 'ssh://$HOSTADDRESS$',
};

$expected = {
    managementurl => [
        {   name => "SSH",
            url  => 'ssh://$HOSTADDRESS$',
        }
    ]
};

is_deeply( Opsview::Web::Utils->convert_multilist_params_to_hash($params), $expected, "Got expected hash with single vars" );

$params = {
    "variables[name]" => [ "DISK", "DISK", "PROCS", "PROCS", "INTERFACE" ],
    "variables[value]" => ["/u01", "/tmp", "init",  "cron",  "eth0"      ],
    "variables[args]" =>  ["",     "-p /tmp -c 5%", "", "-u root -C cron", "-c 90%" ],
    "untouched" => 35,
};
$expected = {
    variables => [ 
        { name => "DISK", value => "/u01", args => "", },
        { name => "DISK", value => "/tmp", args => "-p /tmp -c 5%", },
        { name => "PROCS", value => "init", args => "", },
        { name => "PROCS", value => "cron", args => "-u root -C cron", },
        { name => "INTERFACE", value => "eth0", args => "-c 90%", },
    ]
};
is_deeply( Opsview::Web::Utils->convert_multilist_params_to_hash($params, { delete => 1 } ), $expected, "Got expected hash with multiple args" );
is_deeply( $params, { untouched => 35 }, "And parameters deleted" );


$params = {
    "hostattributes[1][name]" => "DISK",
    "hostattributes[1][value]" => "/root",
    "hostattributes[1][arg1]" => "-w 10% -c 5%",
    "hostattributes[1][arg2]" => undef,
    "hostattributes[1][arg3]" => "helpful",
    "hostattributes[2][name]" => "DISK",
    "hostattributes[2][value]" => "/root2",
    "hostattributes[2][arg1]" => "-w 22% -c 2%",
    "hostattributes[2][arg2]" => undef,
    "hostattributes[2][arg3]" => undef,
};
$expected = {
    hostattributes => {
        1 => { name => "DISK", value => "/root", arg1 => "-w 10% -c 5%",  arg2 => undef, arg3 => "helpful" },
        2 => { name => "DISK", value => "/root2", arg1 => "-w 22% -c 2%", arg2 => undef, arg3 => undef },
    }
};
my $got = Opsview::Web::Utils->convert_indexed_params_to_hash($params);
is_deeply( $got, $expected, "Got expected hash with indexed params" ) || diag explain $got;

