#!/usr/bin/perl
# Test /admin/role pages

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

$mech->get_ok("/admin/role");
$mech->follow_link( text => "Public" );

$mech->content_contains("tab-access", "Found tab for access information");
$mech->content_lacks("tab-objects", "Missing tab for objects, because is a system role" );

$mech->get_ok("/admin/role");
$mech->content_like(qr/View all, change none/);

$mech->follow_link( text => "View all, change none" );

#$mech->get_ok("/admin/role/edit/13");
$mech->content_like(qr/Edit: View all, change none/);
$mech->content_contains("tab-objects", "Has tab for objects here" );
$mech->form_name("main_form");
is( $mech->value("name"),        "View all, change none" );
is( $mech->value("description"), "Read only user" );
is( $mech->value( 'accesses', 1 ),  undef );
is( $mech->value( 'accesses', 2 ),  undef );
is( $mech->value( 'accesses', 3 ),  undef );
is( $mech->value( 'accesses', 4 ),  undef );
is( $mech->value( 'accesses', 5 ),  undef );
is( $mech->value( 'accesses', 6 ),  undef );
is( $mech->value( 'accesses', 7 ),  undef );
is( $mech->value( 'accesses', 8 ),  undef );
is( $mech->value( 'accesses', 9 ),  undef );
is( $mech->value( 'accesses', 10 ),  6 );       # This is NOTIFYSOME for the view all, change none user
is( $mech->value( 'accesses', 11 ),  14, "PASSWORDSAVE" );
is( $mech->value( 'accesses', 12 ),  undef );
is( $mech->value( 'accesses', 13 ),  undef );
is( $mech->value( 'accesses', 14 ),  1 );
is( $mech->value( 'accesses', 15 ),  undef );
is( $mech->value( 'accesses', 16 ), undef );

$mech->field( "name",        "badrobot" );
$mech->field( "description", "New role info" );
$mech->tick( "accesses", 7 );
$mech->tick( "accesses", 10 );
$mech->untick( "accesses", 6 );
$mech->submit_form;

$mech->content_like(qr{/admin/role});    # meta refresh link

$mech->get_ok("/admin/role/edit/13");
$mech->content_like(qr/Edit: badrobot/);
$mech->form_name("main_form");

is( $mech->value("name"),        "badrobot" );
is( $mech->value("description"), "New role info" );
is( $mech->value( 'accesses', 1 ),  undef );
is( $mech->value( 'accesses', 2 ),  undef );
is( $mech->value( 'accesses', 3 ),  undef );
is( $mech->value( 'accesses', 4 ),  7 );
is( $mech->value( 'accesses', 5 ),  undef );
is( $mech->value( 'accesses', 6 ),  undef );
is( $mech->value( 'accesses', 7 ),  undef );
is( $mech->value( 'accesses', 8 ),  undef );
is( $mech->value( 'accesses', 9 ),  undef );
is( $mech->value( 'accesses', 10 ),  undef );
is( $mech->value( 'accesses', 11 ),  14, "PASSWORDSAVE" );
is( $mech->value( 'accesses', 12 ),  undef );
is( $mech->value( 'accesses', 13 ),  undef );
is( $mech->value( 'accesses', 14 ),  1 );
is( $mech->value( 'accesses', 15 ),  10 );
is( $mech->value( 'accesses', 16 ), undef );

# Proves that all accesses can be disabled
$mech->untick( "accesses", 7 );
$mech->untick( "accesses", 10 );
$mech->untick( "accesses", 1 );
$mech->untick( "accesses", 14 );
$mech->submit_form;

$mech->get_ok("/admin/role/edit/13");
$mech->form_name("main_form");
is( $mech->value( 'accesses', 1 ),  undef );
is( $mech->value( 'accesses', 2 ),  undef );
is( $mech->value( 'accesses', 3 ),  undef );
is( $mech->value( 'accesses', 4 ),  undef );
is( $mech->value( 'accesses', 5 ),  undef );
is( $mech->value( 'accesses', 6 ),  undef );
is( $mech->value( 'accesses', 7 ),  undef );
is( $mech->value( 'accesses', 8 ),  undef );
is( $mech->value( 'accesses', 9 ),  undef );
is( $mech->value( 'accesses', 10 ), undef );
is( $mech->value( 'accesses', 11 ), undef );
is( $mech->value( 'accesses', 12 ), undef );
is( $mech->value( 'accesses', 13 ), undef );
is( $mech->value( 'accesses', 14 ), undef );
is( $mech->value( 'accesses', 15 ), undef );
is( $mech->value( 'accesses', 16 ), undef );

$mech->get_ok("/admin/role");
$mech->follow_link( text => "Create new Role" );
$mech->form_name("main_form");
$mech->field( "name",        "new role" );
$mech->field( "description", "A new auto role" );
$mech->tick( "accesses", 7 );
$mech->submit_form;

$mech->get_ok("/admin/role");
$mech->follow_link( text => "new role" );
$mech->form_name("main_form");
is( $mech->value("name"),        "new role" );
is( $mech->value("description"), "A new auto role" );
is( $mech->value( 'accesses', 1 ),  undef );
is( $mech->value( 'accesses', 2 ),  undef );
is( $mech->value( 'accesses', 3 ),  undef );
is( $mech->value( 'accesses', 4 ),  7 );
is( $mech->value( 'accesses', 5 ),  undef );
is( $mech->value( 'accesses', 6 ),  undef );
is( $mech->value( 'accesses', 7 ),  undef );
is( $mech->value( 'accesses', 8 ),  undef );
is( $mech->value( 'accesses', 9 ),  undef );
is( $mech->value( 'accesses', 10 ), undef );
is( $mech->value( 'accesses', 11 ), undef );
is( $mech->value( 'accesses', 12 ), undef );
is( $mech->value( 'accesses', 13 ), undef );
is( $mech->value( 'accesses', 14 ), undef );
is( $mech->value( 'accesses', 15 ), undef );
is( $mech->value( 'accesses', 16 ), undef );

$mech->get_ok("/admin/role/clone/12");
$mech->form_name("main_form");
$mech->field( "name", "clonedrole" );
$mech->submit_form;
$mech->get_ok("/admin/role");
$mech->content_like(qr/View some, change some/);
$mech->follow_link( text => "clonedrole" );
$mech->form_name("main_form");
is( $mech->value("name"),        "clonedrole" );
is( $mech->value("description"), "Restricted operator" );
is( $mech->value( 'accesses', 1 ),  undef );
is( $mech->value( 'accesses', 2 ),  4, "ACTIONSOME" );
is( $mech->value( 'accesses', 3 ),  undef );
is( $mech->value( 'accesses', 4 ),  7, "CONFIGUREHOSTS" );
is( $mech->value( 'accesses', 5 ),  undef );
is( $mech->value( 'accesses', 6 ),  13, "CONFIGURESAVE" );
is( $mech->value( 'accesses', 7 ),  undef );
is( $mech->value( 'accesses', 8 ),  undef );
is( $mech->value( 'accesses', 9 ),  17, "DOWNTIMESOME" );
is( $mech->value( 'accesses', 10 ),  6, "NOTIFYSOME" );
is( $mech->value( 'accesses', 11 ),  14, "PASSWORDSAVE" );
is( $mech->value( 'accesses', 12 ),  undef );
is( $mech->value( 'accesses', 13 ),  undef );
is( $mech->value( 'accesses', 14 ),  undef );
is( $mech->value( 'accesses', 15 ),  undef );
is( $mech->value( 'accesses', 16 ), 2, "VIEWSOME" );

is( $mech->value( "hostgroups", 1 ), undef );
is( $mech->value( "hostgroups", 2 ), undef );
is( $mech->value( "hostgroups", 3 ), undef );
is( $mech->value( "hostgroups", 4 ), 7, "singlehost hostgroup" );
is( $mech->value( "hostgroups", 5 ), undef );
is( $mech->value( "hostgroups", 6 ), undef );
is( $mech->value( "hostgroups", 7 ), undef );
is( $mech->value( "hostgroups", 8 ), 6, "UK2 hostgroup" );
is( $mech->value( "hostgroups", 9 ), undef );
is( $mech->value( "hostgroups", 10 ), undef );

is( $mech->value( "monitoringservers", 1 ), undef );
is( $mech->value( "monitoringservers", 2 ), undef, "ClusterA" );


$mech->get_ok("/admin/role/edit/18");
$mech->form_name("main_form");
is( $mech->value( "monitoringservers", 2 ), 2, "ClusterA" );


$mech->get_ok("/admin/role");
@_ = $mech->find_link( url_regex => qr^/admin/role/delete/12^ );
is_deeply( \@_, [], "No delete option for 'View some, change some' because is currently in use" );

my $link = $mech->find_link( url_regex => qr^/admin/role/delete^ );
my $url_delete = $link->url;

$mech->get( "/admin/role/delete/12", "This should fail" );
is( $mech->status, 500, "Web exception" );
$mech->content_like( qr/Cannot delete or update a parent row/, "webexception due to delete" );

$mech->get_ok( $link, "Deleted first available role: $url_delete" );
$mech->get_ok("/admin/role");
$mech->content_contains( "Create new Role", "Back to list page" );
@_ = $mech->find_link( url => $url_delete );
is_deeply( \@_, [], "Role deleted: $url_delete" );


# Keyword tests, migrated from contact form
my $keyword_lookup = {
    "allhandled" => 9,
    "allhosts"   => 7,
    "allservicechecks" => 8,
    "alphaearly" => 6,
    "cisco"      => 2,
    "cisco_gp1"  => 3,
    "cisco_gp2"  => 4,
    "cloneable"  => 1,
    "disabled"   => 5,
};

sub check_ticks {
    my ( $ua, $field, $expected ) = @_;
    for my $i ( sort keys %$expected ) {
        my $want = $expected->{$i};
        if ( defined $want ) {
            $want = $keyword_lookup->{$want};
        }
        is( $ua->value( $field, $i ), $want, "Field $field at $i = " . Data::Dump::dump( $expected->{$i} ) );
    }
}

my $content;
my $ua = $mech;

$ua->get_ok("/admin/role");
$ua->get_ok("/admin/role/edit/10");
$ua->form_name("main_form");
$ua->click_button( value => "Submit Changes" );
$ua->content_like( qr^URL=http://localhost/admin/role^, "Redirected back to list page" );

$ua->get_ok("/admin/role");
$ua->get_ok("/admin/role/new");
$ua->form_name("main_form");
$ua->field( "name", "test455" );
$ua->click_button( value => "Submit Changes" );
$ua->content_like( qr^URL=http://localhost/admin/role^, "Redirected back to list page on create" );

$ua->get_ok("/admin/role/edit/10");

$ua->form_name("main_form");
# This is ordered top to bottom, left to right on the page - take a look to work out the numbers
my $expected = {
    1 => undef,
    2 => undef,
    3 => undef,
    4 => "allhosts",
    5 => undef,
    6 => undef,
    7 => "allservicechecks",
    8 => undef,
    9 => undef,
};
check_ticks( $ua, "access_keywords", $expected );

foreach my $i ( 1 .. (scalar keys %$keyword_lookup) ) {
    $ua->tick( "access_keywords", $i );
}
$ua->current_form->action;
$ua->submit;
ok( $ua->res->is_success, "Saved with all keywords" );
$ua->get_ok("/admin/role/edit/10");
$ua->form_name("main_form");
$expected = {
    1 => "allhandled",
    2 => "alphaearly",
    3 => "cisco_gp2",
    4 => "allhosts",
    5 => "cisco",
    6 => "cloneable",
    7 => "allservicechecks",
    8 => "cisco_gp1",
    9 => "disabled",
};
check_ticks( $ua, "access_keywords", $expected );

foreach my $i ( 1 .. (scalar keys %$keyword_lookup) ) {
    $ua->untick( "access_keywords", $i );
}
$ua->tick( "access_keywords", 3 );
$ua->current_form->action;
$ua->submit;
ok( $ua->res->is_success, "Saved with 1 keyword" );
$ua->get_ok("/admin/role/edit/10");
$ua->form_name("main_form");
$expected = {
    1 => undef,
    2 => undef,
    3 => undef,
    4 => undef,
    5 => undef,
    6 => undef,
    7 => undef,
    8 => "cisco_gp1",
    9 => undef,
};
check_ticks( $ua, "access_keywords", $expected );

$ua->untick( "access_keywords", 3 );
$ua->tick( "access_keywords", 5 );
$ua->tick( "access_keywords", 6 );
$ua->current_form->action;
$ua->submit;

ok( $ua->res->is_success, "Saved with 2 keywords" );

$ua->get_ok("/admin/role/edit/10");
$ua->form_name("main_form");
$expected = {
    1 => undef,
    2 => "alphaearly",
    3 => undef,
    4 => undef,
    5 => undef,
    6 => undef,
    7 => undef,
    8 => undef,
    9 => "disabled",
};
check_ticks( $ua, "access_keywords", $expected );

$ua->untick( "access_keywords", 5 );
$ua->untick( "access_keywords", 6 );
$ua->current_form->action;
$ua->submit;

ok( $ua->res->is_success, "Saved with 0 keywords" );

$ua->get_ok("/admin/role/edit/10");
$ua->form_name("main_form");
$expected = {
    1 => undef,
    2 => undef,
    3 => undef,
    4 => undef,
    5 => undef,
    6 => undef,
    7 => undef,
    8 => undef,
    9 => undef,
};
check_ticks( $ua, "access_keywords", $expected );

