#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $content;
my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

$mech->get_ok("/admin/host/query_host?hostid=1&ip=bob&msid=1&snmp_community=public&snmp_version=2c");
my $url = "/admin/host/query_host?hostid=1&msid=1&snmp_community=public&snmp_version=2c";

Opsview::Web::Controller::Admin::Host->config( query_host_command => "$Bin/bin/noexec" );
$mech->get_ok("$url&ip=bob");
$mech->content_like(qr^<span class="error_ignoreable">Cannot query host: permission denied: .*</span>^);

Opsview::Web::Controller::Admin::Host->config( query_host_command => "$Bin/bin/query_host_dummy" );
$mech->get_ok("$url&ip=nooutput");
$mech->content_like(qr^<span class="error_ignoreable">query_host output is empty</span>^);

$mech->get_ok("$url&ip=badxml");
$mech->content_like(qr^<span class="error_ignoreable">query_host returned invalid XML: &lt;bad&gt;.*</span>^s);

$mech->get_ok("$url&ip=errormessage");
$mech->content_like(qr^<span class="error_ignoreable">We have a problem</span>^);

$mech->get_ok("$url&ip=nointerfaces");
$mech->content_like(qr^<span class="error_ignoreable">No interfaces found - click Query Host to start scanning for interfaces</span>^s);

$mech->get_ok("$url&ip=opsviewdev46");
$mech->content_like(qr%<span class="error_ignoreable"></span>%, "No errors listed" );
$content = $mech->content;
my @occurances;
@occurances = ($content =~ /snmpinterfaces_all/g);
is( scalar @occurances, 5, "Found 4 + 1 interfaces on page" );
$mech->content_like(qr/Linux opsviewdev46 2.6.18-6-686 #1 SMP Tue Jun 17 21:31:27 UTC 2008 i686/, "Got sysdescr too" );

$mech->get_ok("$url&ip=singleinterface");
$mech->content_like(qr%<span class="error_ignoreable"></span>%, "No errors listed" );
$content = $mech->content;
@occurances = ($content =~ /snmpinterfaces_all/g);
is( scalar @occurances, 2, "Found 1 + 1 interfaces on page" );

$mech->get_ok("$url&ip=okwithwarning");
$mech->content_like(qr^<span class="error_ignoreable">Cannot connect with SNMPv2c: No response from remote host 'localhost'. Trying with SNMPv1</span>^);

$mech->get_ok("$url&ip=errorwithwarning");
$mech->content_like(qr^<span class="error_ignoreable">Cannot get sysDescr - check community string: No response from remote host 'localhost'</span>^);
$mech->content_unlike(qr^Cannot connect^);

$mech->get_ok("$url&ip=multierrors");
$mech->content_like(qr^<span class="error_ignoreable">Bad; Man</span>^);

$mech->get_ok("$url&ip=multiwarnings");
$mech->content_like(qr^<span class="error_ignoreable">Holy mackrel; Batman</span>^);

$mech->get_ok("$url&ip=emptyifdescr");
$mech->content_like(qr^<span class="error_ignoreable">Blank interface name for interface id 4 - ignoring</span>^);

$mech->get_ok("http://localhost/state/host/998/queryhost");
$mech->content_like(qr/Host does not exist in configuration/);

sub strip_docs_version {
    $_ = shift;
    s/opsview\d+\.\d+://g;
    s/opsview-community://g;
    $_;
}
