#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;


use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );
witter('Checking downtime detail on "Monitoring Servers" hostgroup');

$mech->get_ok('http://localhost/state/hostgroup/3/detail');
$mech->content_like(qr/Detail for UK/);
$mech->get_ok('http://localhost/status/service?hostgroupid=3');
$mech->content_like(qr/Service Status &gt; Host Group: UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_hosts');
$mech->content_like(qr/Hosts with Scheduled Downtime within UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_services');
$mech->content_like(qr/Services with Scheduled Downtime within UK/);

# Check for <script> tags
my @links = qw(
	/state/hostgroup/6/detail
	/state/hostgroup/6/downtime/list_downtime_hosts
	/state/hostgroup/6/downtime/list_downtime_services
	/state/host/136/detail
	/state/service/193/detail
	/status/service?downtime_comment=Hostgroup+'Leaf2'%3A+in+the+future+%3Cscript%3Ealert(%22badness%22)%3C%2Fscript%3E&downtime_start_time=2017-11-20+14%3A34%3A47
);
foreach my $link (@links) {
	$mech->get_ok($link);
	$mech->content_contains("Hostgroup 'Leaf2': in the future");
	$mech->content_lacks('<script>alert("badness")</script>');
	$mech->content_contains('&lt;script&gt;alert(&quot;badness', "Escaped characters");
}

log_out($mech);

log_in( $mech, 'nonadmin', 'nonadmin' );

$mech->get_ok('http://localhost/state/hostgroup/3/detail');
$mech->content_like(qr/Detail for UK/);
$mech->get_ok('http://localhost/status/service?hostgroupid=3');
$mech->content_like(qr/Service Status &gt; Host Group: UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_hosts');
$mech->content_like(qr/Hosts with Scheduled Downtime within UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_services');
$mech->content_like(qr/Services with Scheduled Downtime within UK/);

log_out($mech);

{
    local $TODO = "somehosts shouldnt have access to hsotgroup 3 stuff really";
    log_in( $mech, 'somehosts', 'somehosts' );

    $mech->get_ok('http://localhost/state/hostgroup/3/detail');
    $mech->content_like(qr/Detail for UK/);
    $mech->get_ok('http://localhost/status/service?hostgroupid=3');
    $mech->content_like(qr/Service Status for Host Group UK/);
    $mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_hosts');
    $mech->content_like(qr/Hosts with Scheduled Downtime within UK/);
    $mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_services');
    $mech->content_like(qr/Services with Scheduled Downtime within UK/);

    log_out($mech);
}

log_in( $mech, 'readonly', 'readonly' );

$mech->get_ok('http://localhost/state/hostgroup/3/detail');
$mech->content_like(qr/Detail for UK/);
$mech->get_ok('http://localhost/status/service?hostgroupid=3');
$mech->content_like(qr/Service Status &gt; Host Group: UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_hosts');
$mech->content_like(qr/Hosts with Scheduled Downtime within UK/);
$mech->get_ok('http://localhost/state/hostgroup/3/downtime/list_downtime_services');
$mech->content_like(qr/Services with Scheduled Downtime within UK/);

log_out($mech);
