#!/usr/bin/perl
# Test setting downtimes

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

# Do a service downtime
$mech->get_ok("/status/service?host=opslave");
$mech->title_is("Opsview > Service Status > Host: opslave");
$mech->content_unlike( qr/downtime\.png/, "No downtime icon here" );

$mech->get_ok("/state/service/197/downtime");
$mech->title_is("Opsview > Schedule Downtime");
$mech->content_contains( "Schedules downtime for the service / on host opslave", "Schedule downtime page" );
$mech->form_name("main_form");
$mech->field( "comment", "test downtime by mech" );
$mech->current_form->action;
$mech->submit;

$mech->title_is("Opsview > Service Status > Host: opslave");
$mech->content_contains("Downtime submitted for service / on host opslave");

# Do a host downtime
$mech->get_ok("/status/host?hostgroupid=2");
$mech->title_is("Opsview > Host Summary > Host Group: Monitoring Servers");
$mech->content_unlike( qr/downtime\.png/, "No downtime icon here" );

$mech->get_ok("/state/host/137/downtime");
$mech->title_is("Opsview > Schedule Downtime");
$mech->content_contains( "Schedules downtime for the host opslave", "Schedule downtime page" );
$mech->form_name("main_form");
$mech->field( "comment", "test downtime by mech host" );
$mech->current_form->action;
$mech->submit;

$mech->title_is("Opsview > Host Summary > Host Group: Monitoring Servers");
$mech->content_contains("Downtime submitted for opslave");

# Do a host group downtime
$mech->get_ok("/status/hostgroup/6");
$mech->title_is("Opsview > Host Group Summary > UK2");

$mech->get_ok("/state/hostgroup/5/downtime");
$mech->title_is("Opsview > Schedule Downtime");
$mech->content_contains( "Schedules downtime for Leaf2 and all the hosts and services contained", "Schedule downtime page" );
$mech->form_name("main_form");
$mech->field( "comment", "test downtime by mech hostgroup" );
$mech->current_form->action;
$mech->submit;

$mech->title_is("Opsview > Host Group Summary > UK2");
$mech->content_contains("Downtime submitted for Leaf2");

my @cmds = Opsview::TestUtils->nagioscmds;
is( scalar @cmds, 5 );
like( $cmds[0], qr%SCHEDULE_SVC_DOWNTIME;opslave;/;\d+;\d+;1;0;;admin;Service '/' on host 'opslave': test downtime by mech% );
like( $cmds[1], qr%SCHEDULE_HOST_SVC_DOWNTIME;opslave;\d+;\d+;1;0;;admin;Host 'opslave': test downtime by mech host% );
like( $cmds[2], qr%SCHEDULE_HOST_DOWNTIME;opslave;\d+;\d+;1;0;;admin;Host 'opslave': test downtime by mech host% );
like( $cmds[3], qr%SCHEDULE_HOSTGROUP_SVC_DOWNTIME;Leaf2;\d+;\d+;1;0;;admin;Host group 'Leaf2': test downtime by mech hostgroup% );
like( $cmds[4], qr%SCHEDULE_HOSTGROUP_HOST_DOWNTIME;Leaf2;\d+;\d+;1;0;;admin;Host group 'Leaf2': test downtime by mech hostgroup% );
